#!/bin/python3
#%%
import json
import re

#%%

def verbose_print(string: str, verbose = False):
    if verbose:
        print(string)

#%%

def read_file_with_fallback(file_path, encoding = 'auto', verbose = False):
    verbose_print(f'[read_file_with_fallback] read file_path="{file_path}"', verbose=verbose)
    if encoding == 'auto':
        encodings = ['utf-8', 'gbk', 'big5']
    else:
        encodings = [encoding]
    content = None
    correct_encoding = None
    for encoding in encodings:
        try:
            with open(file_path, 'r', encoding=encoding) as f:
                content = f.read()
                correct_encoding = encoding
            verbose_print(f"[read_file_with_fallback] Successfully decoded using {encoding}", verbose=verbose)
            break
        except UnicodeDecodeError:
            verbose_print(f"[read_file_with_fallback] Failed to decode using {encoding}", verbose=verbose)
    return content, correct_encoding


class TextFrames(object):

    def __init__(self, string = None, path = None, encoding = 'auto', verbose = False):

        verbose_print('[TextFrames][__init__]', verbose=verbose)

        self.string = None
        self.path = None
        self.verbose = verbose

        if string is not None and isinstance(string, str):
            self.string = string
            self.code = None
            self.path = None
        elif path is not None :
            self.string, self.code = read_file_with_fallback(path, encoding=encoding)
            self.path = path
        else:
            raise ValueError('[TextFrames][__init__] Input path is not None and isinstance(path, str)')
        
        return None
    
    def __str__(self):
        return self.string
    
    def __repr__(self):
        return self.string
    
    def __len__(self):
        return len(self.string)

    def __getitem__(self, index, sep = '\n'):
        return self.string.split(sep)[index]
    
    def replace(self, old, new, regrex = False, inplace = False, validate = None):

        old = str(old)
        new = str(new)

        if validate not in [None, 'unique', 'multiple']:
            raise ValueError(f'[TextFrames][replace] validate not in [None, unique, multiple]')

        if regrex:
            match_list = re.findall(old, self.string)
            if validate == 'unique':
                if len(match_list) != 1:
                    raise ValueError(f'[TextFrames][replace] validate = {validate} and match_list = {match_list}')
            elif validate == 'multiple':
                if len(match_list) == 0:
                    raise ValueError(f'[TextFrames][replace] validate = {validate} and match_list = {match_list}')
            
            new_string = re.sub(old, new, self.string)
            verbose_print(f'[TextFrames][replace] {old} -> {new}, {len(match_list)} matches found', verbose=self.verbose)

        else:
            count = self.string.count(old)
            if validate == 'unique':
                if count != 1:
                    raise ValueError(f'[TextFrames][replace] validate = {validate} and self.string.count(old) = {count}')
            elif validate == 'multiple':
                if count == 0:
                    raise ValueError(f'[TextFrames][replace] validate = {validate} and self.string.count(old) = {count}')
                
            new_string = self.string.replace(old, new)
            verbose_print(f'[TextFrames][replace] {old} -> {new}, {count} matches found', verbose=self.verbose)
        
        if inplace:
            self.string = new_string
            result = None
        else:
            new_TextFrames = TextFrames(string=new_string, path=self.path)
            result = new_TextFrames

        return result
    

    def counts(self, pattern, regrex = False ):
        pattern = str(pattern)
        if regrex:
            match_list = re.findall(pattern, self.string)
            result = len(match_list)
        else:
            result = self.string.count(pattern)
        return result
    

    def contains(self, pattern, regrex = False, validate = 'multiple', error = 'print'):

        count = self.counts(pattern=pattern, regrex=regrex)
        log_str = f'[TextFrames][contains] pattern = "{pattern}", validate = {validate}, count = {count}'

        result = False
        if validate == 'unique':
            if count == 1:
                result = True
        elif validate == 'multiple':
            if count > 0:
                result = True
        
        if not result:
            if error == 'print':
                verbose_print(log_str, verbose=self.verbose)
            elif error == 'raise':
                raise ValueError(log_str)
            
        return result
    

    def to_list(self, sep = '\n', drop_space = False):
        s = self.string.split(sep)
        if drop_space:
            s = [k.strip() for k in s if k]
        return s
    

    def json_loads(self):
        return json.loads(self.string)

    def to_txt(self, path = None, encoding = 'utf-8'):
        if path is None:
            path = self.path
        with open(path, 'w', encoding=encoding) as f:
            f.write(self.string)
        return None
    
    save = to_txt
    
    pass
    # end of class
